/*
 *  PHunterLib.cp
 *  PHunterLib
 *
 *  Created by Tiayyba Riaz on 6/7/09.
 *  Copyright 2009 __MyCompanyName__. All rights reserved.
 *
 */

#include <stdlib.h>
#include "libthermo.h"
#include "nnparams.h"
#include "thermalign.h"
#include "galign.h"
#include "dinkelbach.h"


//#define EXPORT __attribute__((visibility("default")))

//char targetsFile [1000]="";
//char nonTargetsFile [1000]="";
//char primersFile [1000]="";
//char primersOutputFile [1000]="";
//int minLength = DEF_MIN_PRIMER_LENGTH;
//int maxLength = DEF_MAX_PRIMER_LENGTH;
//int productMinLength = DEF_MIN_PROD_LENGTH;
//int productMaxLength = DEF_MAX_PROD_LENGTH;
//char forwardPrimer [PRIMERS_MAX_SIZE] = DEF_FORWARD_PRIMER;
//char reversePrimer [PRIMERS_MAX_SIZE] = DEF_REVERSE_PRIMER;
//int beginPosForward = DEF_BEGINPOS_FORWARD;
//int endPosForward = DEF_ENDPOS_FORWARD;
//int beginPosReverse = DEF_BEGINPOS_REVERSE;
//int endPosReverse = DEF_ENDPOS_REVERSE;
//char maskTargets [PRIMERS_MAX_SIZE] = DEF_MASK_TARGETS;
//char maskNonTargets [PRIMERS_MAX_SIZE] = DEF_MASK_NONTARGETS;
//char degeneracy[PRIMERS_MAX_SIZE]=DEF_DEGENERACY;
//int nseqForCandidates = DEF_SEQ_FOR_CANDS;
//float minCoverageTargets = DEF_MIN_COVERAGE_TARGETS;
//float maxCoverageNonTargets = DEF_MAX_COVERAGE_NONTARGETS;
//float maxSelfScore = DEF_MAX_SELF_SCORE;
//float maxEndScore = DEF_MAX_END_SCORE;
//float minGCContent = DEF_MIN_GCCONTENT; 
//float maxGCContent = DEF_MAX_GCCONTENT;
//int gcClamp = DEF_GCCLAMP;
//int maxPolyX = DEF_MAX_POLY_X;
float concPrimers = DEF_CONC_PRIMERS; 
float concSequences = DEF_CONC_SEQUENCES; 
float salt = DEF_SALT;
int salMethod = SALT_METHOD_SANTALUCIA; 
//float minTempTargets=DEF_MIN_TEMP_TARGETS;
//float maxTempTargets=DEF_MAX_TEMP_TARGETS;
//float maxTempNonTargets=DEF_MAX_TEMP_NONTARGETS;
//float deltaTempNonTargets=DEF_DELTA_TEMP_NONTARGETS;
//float maxPairTempDiff=DEF_MAX_PAIR_TEMP_DIFF;
//char primersLabel [1000] = DEF_PRIMERS_LABEL;
//int fullStats = DEF_FULL_STATS;
//char seqRank [10000] = DEF_SEQ_RANK;


int countGCContent(char * seq ) {
	int lseq = strlen(seq);
	int k;
	int count = 0;
	for( k=0;k<lseq;k++) {
		if (seq[k] == 'G' || seq[k] == 'C' ) {
			count+=1;
		}
	}
	return count;
}


char getComplement(char base) {
	if(base == 'A') return 'T';
	else if(base == 'G') return 'C';
	else if(base == 'C') return 'G';
	else if(base == 'T') return 'A';
	else if(base == 'N') return 'N';
	return 'A';
}


void addDollarSigns(char * seq) {
	int n = strlen(seq);
	int i;
	
	seq[n+2]=0;
	seq[n+1]='$';
	for( i=n;i>0;i--) {
		seq[i]=seq[i-1];
	}
	seq[0] = '$';
}

void removeDollarSigns(char * seq) {
	int n = strlen(seq);
	int i;
	for( i=0;i<n-2;i++) {
		seq[i]=seq[i+1];
	}
	seq[n-2]=0;
}

float calculateMeltingTemperatureInt (char * seq1, char * seq2, PNNParams params) {
	int length = strlen(seq2);
	int perfectComplements = 1;
	int k;
	
	if(strlen(seq1)!= length) {
		perfectComplements = 0;
	} 
	for( k=0;k<length && perfectComplements;k++) {
		if(seq2[k] != getComplement(seq1[k])) {
			perfectComplements = 0;
		}
	}
	addDollarSigns(seq1);
	addDollarSigns(seq2);
	
	CThermAlign myAlignment;//(strlen(seq1),strlen(seq2),params);
	therm_initCThermAlign(&myAlignment, strlen(seq1),strlen(seq2),params);
	
	therm_InitStrings (&myAlignment, seq1,seq2,strlen(seq1),strlen(seq2), 0);
	
	therm_InitBorder(&myAlignment);
	
	therm_CalculateTable(&myAlignment, 1);
	
	float answer;
	if(perfectComplements) {
		answer = therm_GetMeltingTempC(&myAlignment, myAlignment.maxloci,myAlignment.maxlocj);
		
	} else {
		float tempK = therm_GetMeltingTempK (&myAlignment, myAlignment.maxloci,myAlignment.maxlocj);
		
		GAlign mydGAlign;//(strlen(seq1),strlen(seq2),params);
		galn_initGAlign(&mydGAlign, strlen(seq1),strlen(seq2),params);
		
		galn_InitStrings(&mydGAlign, seq1,seq2,strlen(seq1),strlen(seq2), 0);
		
		dinkelbach myDinkel;//(params, &mydGAlign);
		dnkl_initdinkelbach(&myDinkel, params, &mydGAlign);
		
		dnkl_iteration(&myDinkel, tempK);
		
		answer =  galn_GetMeltingTempC(&mydGAlign, mydGAlign.maxloci,mydGAlign.maxlocj);
		
		galn_finiGAlign(&mydGAlign);
	}
	removeDollarSigns(seq1);
	removeDollarSigns(seq2);
	
	therm_finiCThermAlign(&myAlignment);
	return answer;
}

float calculateMeltingTemperature (char * seq1, char * seq2) {
	
	long len = strlen (seq1);
	char *seq1new = (char *) calloc(len + 3, sizeof(char));//new char[len + 3];
	len = strlen (seq2);
	char *seq2new = (char *) calloc(len + 3, sizeof(char));//new char[len + 3];
	
	strcpy (seq1new, seq1);
	strcpy (seq2new, seq2);
	
	PNNParams paramsPrimerSeq = (PNNParams) calloc (1, sizeof(CNNParams));
	nparam_InitParams(paramsPrimerSeq, concPrimers,concSequences,salt,salMethod);
  
	float temp = calculateMeltingTemperatureInt(seq1new, seq2new, paramsPrimerSeq);
	
	free (seq1new);
	free (seq2new);
	
	return temp;
}

float calculateMeltingTemperatureBasic (char * seq) {
	int gccount;
	float temp;
	int seqlen;
	
	seqlen = strlen (seq);
	gccount = countGCContent (seq);
	temp = 64.9 + 41*(gccount - 16.4)/seqlen;
	return temp;
}



